@@ -30,7 +30,10 @@
 #include <EntityDisplayAttribute>
 #include <EntityTreeModel>
 #include <Item>
+
+#include <AkonadiCore/AgentInstance>
 #include <AkonadiCore/AgentManager>
+#include <AkonadiCore/EntityDisplayAttribute>
 #include <Akonadi/Calendar/ETMCalendar>
 #include <Akonadi/Calendar/PublishDialog>
 #include <akonadi/calendar/calendarsettings.h>
@@ -64,6 +67,7 @@
 #include <QPointer>
 #include <QFileDialog>
 #include <QUrlQuery>
+#include <QRegularExpression>
 #include <QTimeZone>
 
 #include "calendarsupport_debug.h"
@@ -77,7 +81,7 @@ KCalCore::Incidence::Ptr CalendarSupport::incidence(const Akonadi::Item &item)
     //relying on exception for performance reasons
     try {
         return item.payload<KCalCore::Incidence::Ptr>();
-    } catch (Akonadi::PayloadException) {
+    } catch (const Akonadi::PayloadException &) {
         return KCalCore::Incidence::Ptr();
     }
 }
@@ -90,7 +94,7 @@ KCalCore::Event::Ptr CalendarSupport::event(const Akonadi::Item &item)
         if (hasEvent(incidence)) {
             return item.payload<KCalCore::Event::Ptr>();
         }
-    } catch (Akonadi::PayloadException) {
+    } catch (const Akonadi::PayloadException &) {
         return KCalCore::Event::Ptr();
     }
     return KCalCore::Event::Ptr();
@@ -122,7 +126,7 @@ KCalCore::Todo::Ptr CalendarSupport::todo(const Akonadi::Item &item)
         if (hasTodo(incidence)) {
             return item.payload<KCalCore::Todo::Ptr>();
         }
-    } catch (Akonadi::PayloadException) {
+    } catch (const Akonadi::PayloadException &) {
         return KCalCore::Todo::Ptr();
     }
     return KCalCore::Todo::Ptr();
@@ -143,7 +147,7 @@ KCalCore::Journal::Ptr CalendarSupport::journal(const Akonadi::Item &item)
         if (hasJournal(incidence)) {
             return item.payload<KCalCore::Journal::Ptr>();
         }
-    } catch (Akonadi::PayloadException) {
+    } catch (const Akonadi::PayloadException &) {
         return KCalCore::Journal::Ptr();
     }
     return KCalCore::Journal::Ptr();
@@ -540,15 +544,13 @@ QString CalendarSupport::displayName(Akonadi::ETMCalendar *calendar, const Akona
                     nameStr = cName;
                     typeStr = i18n("Calendar");
                     break;
-                } else if (tName.startsWith(QLatin1String("shared.tasks"), Qt::CaseInsensitive)
-                           || tName.startsWith(QLatin1String("shared.todo"),
-                                               Qt::CaseInsensitive)) {
+                } else if (tName.startsWith(QLatin1String("shared.tasks"), Qt::CaseInsensitive) ||
+                           tName.startsWith(QLatin1String("shared.todo"), Qt::CaseInsensitive)) {
                     ownerStr = QStringLiteral("Shared");
                     nameStr = cName;
                     typeStr = i18n("Tasks");
                     break;
-                } else if (tName.startsWith(QLatin1String("shared.journal"),
-                                            Qt::CaseInsensitive)) {
+                } else if (tName.startsWith(QLatin1String("shared.journal"), Qt::CaseInsensitive)) {
                     ownerStr = QStringLiteral("Shared");
                     nameStr = cName;
                     typeStr = i18n("Journal");
@@ -558,10 +560,10 @@ QString CalendarSupport::displayName(Akonadi::ETMCalendar *calendar, const Akona
                     nameStr = cName;
                     typeStr = i18n("Notes");
                     break;
-                } else if (tName != i18n("Calendar")
-                           && tName != i18n("Tasks")
-                           && tName != i18n("Journal")
-                           && tName != i18n("Notes")) {
+                } else if (tName != i18n("Calendar") &&
+                           tName != i18n("Tasks") &&
+                           tName != i18n("Journal") &&
+                           tName != i18n("Notes")) {
                     ownerStr = tName;
                     break;
                 } else {
@@ -576,9 +578,9 @@ QString CalendarSupport::displayName(Akonadi::ETMCalendar *calendar, const Akona
             if (!ownerStr.compare(QLatin1String("INBOX"), Qt::CaseInsensitive)) {
                 return i18nc("%1 is folder contents",
                              "My Kolab %1", typeStr);
-            } else if (!ownerStr.compare(QLatin1String("SHARED"), Qt::CaseInsensitive)
-                       || !ownerStr.compare(QLatin1String("CALENDAR"), Qt::CaseInsensitive)
-                       || !ownerStr.compare(QLatin1String("RESOURCES"), Qt::CaseInsensitive)) {
+            } else if (!ownerStr.compare(QLatin1String("SHARED"), Qt::CaseInsensitive) ||
+                       !ownerStr.compare(QLatin1String("CALENDAR"), Qt::CaseInsensitive) ||
+                       !ownerStr.compare(QLatin1String("RESOURCES"), Qt::CaseInsensitive)) {
                 return i18nc("%1 is folder name, %2 is folder contents",
                              "Shared Kolab %1 %2", nameStr, typeStr);
             } else {
@@ -660,6 +662,52 @@ QString CalendarSupport::displayName(Akonadi::ETMCalendar *calendar, const Akona
     }
 }
 
+QString CalendarSupport::toolTipString(const Akonadi::Collection &coll, bool richText)
+{
+    Q_UNUSED(richText);
+
+    QString str = QLatin1String("<qt>");
+
+    // Display Name
+    QString displayName;
+    if (coll.hasAttribute<Akonadi::EntityDisplayAttribute>()) {
+        displayName = coll.attribute<Akonadi::EntityDisplayAttribute>()->displayName();
+    }
+
+    if (displayName.isEmpty()) {
+        displayName = coll.name();
+    }
+    str += QLatin1String("<b>") + displayName + QLatin1String("</b>");
+    str += QLatin1String("<hr>");
+
+    // Calendar Type
+    QString calendarType;
+    if (!coll.isVirtual()) {
+        const Akonadi::AgentInstance instance =
+            Akonadi::AgentManager::self()->instance(coll.resource());
+        calendarType = instance.type().name();
+    } else {
+        calendarType = i18nc("unknown calendar type", "unknown");
+    }
+    str += QLatin1String("<i>") + i18n("Calendar type:") + QLatin1String("</i>");
+    str += QLatin1String("&nbsp;") + calendarType;
+
+    // Read only?
+    bool isReadOnly = !(coll.rights() & Akonadi::Collection::CanChangeItem);
+    str += QLatin1String("<br>");
+    str += QLatin1String("<i>") + i18n("Rights:") + QLatin1String("</i>");
+    str += QLatin1String("&nbsp;");
+    if (isReadOnly) {
+        str += i18nc("the calendar is read-only", "read-only");
+    } else {
+        str += i18nc("the calendar is read and write", "read+write");
+    }
+    str += QLatin1String("</br>");
+
+    str += QLatin1String("</qt>");
+    return str;
+ }
+
 QString CalendarSupport::subMimeTypeForIncidence(const KCalCore::Incidence::Ptr &incidence)
 {
     return incidence->mimeType();
@@ -680,16 +728,17 @@ QList<QDate> CalendarSupport::workDays(const QDate &startDate, const QDate &endD
     }
 
     if (KCalPrefs::instance()->mExcludeHolidays) {
-        // NOTE: KOGlobals, where this method comes from, used to hold a pointer to
-        //       a KHolidays object. I'm not sure about how expensive it is, just
-        //       creating one here.
-        const HolidayRegion holidays(KCalPrefs::instance()->mHolidays);
-        const Holiday::List list = holidays.holidays(startDate, endDate);
-        const int listCount(list.count());
-        for (int i = 0; i < listCount; ++i) {
-            const Holiday &h = list.at(i);
-            if (h.dayType() == Holiday::NonWorkday) {
-                result.removeAll(h.observedStartDate());
+        foreach (const QString &regionStr, KCalPrefs::instance()->mHolidays) {
+            KHolidays::HolidayRegion region(regionStr);
+            if (region.isValid()) {
+                const KHolidays::Holiday::List list = region.holidays(startDate, endDate);
+                const int listCount(list.count());
+                for (int i = 0; i < listCount; ++i) {
+                    const Holiday &h = list.at(i);
+                    if (h.dayType() == Holiday::NonWorkday) {
+                        result.removeAll(h.observedStartDate());
+                    }
+                }
             }
         }
     }
@@ -701,13 +750,41 @@ QStringList CalendarSupport::holiday(const QDate &date)
 {
     QStringList hdays;
 
-    const HolidayRegion holidays(KCalPrefs::instance()->mHolidays);
-    const Holiday::List list = holidays.holidays(date);
-    const int listCount = list.count();
-    hdays.reserve(listCount);
-    for (int i = 0; i < listCount; ++i) {
-        hdays.append(list.at(i).name());
+    bool showCountryCode = (KCalPrefs::instance()->mHolidays.count() > 1);
+    foreach (const QString &regionStr, KCalPrefs::instance()->mHolidays) {
+        KHolidays::HolidayRegion region(regionStr);
+        if (region.isValid()) {
+            const Holiday::List list = region.holidays(date);
+            const int listCount = list.count();
+            for (int i = 0; i < listCount; ++i) {
+                // don't add duplicates.
+                // TODO: won't find duplicates in different languages however.
+                const QString name = list.at(i).name();
+                if (showCountryCode) {
+                    // If more than one holiday region, append the country code to the holiday
+                    // display name to help the user identify which region it belongs to.
+                    const QRegularExpression holidaySE(
+                        i18nc("search pattern for holidayname", "^%1", name));
+                    if (hdays.filter(holidaySE).isEmpty()) {
+                        const QString pholiday = i18n("%1 (%2)", name, region.countryCode());
+                        hdays.append(pholiday);
+                    } else {
+                        // More than 1 region has the same holiday => remove the country code
+                        // i.e don't show "Holiday (US)" and "Holiday(FR)"; just show "Holiday".
+                        const QRegularExpression holidayRE(
+                            i18nc("replace pattern for holidayname (countrycode)", "^%1 \\(.*\\)", name));
+                        hdays.replaceInStrings(holidayRE, name);
+                        hdays.removeDuplicates();
+                    }
+                } else {
+                    if (!hdays.contains(name)) {
+                        hdays.append(name);
+                    }
+                }
+            }
+        }
     }
+
     return hdays;
 }
 
